/// <reference types="node" />
import { RedisReply } from './commands';
export interface QueueCommandOptions {
    asap?: boolean;
    signal?: any;
    chainId?: symbol;
}
export declare type CommandsQueueExecutor = (encodedCommands: string) => boolean | undefined;
export declare enum PubSubSubscribeCommands {
    SUBSCRIBE = "SUBSCRIBE",
    PSUBSCRIBE = "PSUBSCRIBE"
}
export declare enum PubSubUnsubscribeCommands {
    UNSUBSCRIBE = "UNSUBSCRIBE",
    PUNSUBSCRIBE = "PUNSUBSCRIBE"
}
export declare type PubSubListener = (message: string, channel: string) => unknown;
export declare type PubSubListenersMap = Map<string, Set<PubSubListener>>;
export default class RedisCommandsQueue {
    #private;
    get waitingToBeSentCommandsLength(): number;
    constructor(maxLength: number | null | undefined, executor: CommandsQueueExecutor);
    addEncodedCommand<T = RedisReply>(encodedCommand: string, options?: QueueCommandOptions): Promise<T>;
    subscribe(command: PubSubSubscribeCommands, channels: string | Array<string>, listener: PubSubListener): Promise<void>;
    unsubscribe(command: PubSubUnsubscribeCommands, channels?: string | Array<string>, listener?: PubSubListener): Promise<void>;
    resubscribe(): Promise<any> | undefined;
    executeChunk(recommendedSize: number): boolean | undefined;
    parseResponse(data: Buffer): void;
    flushWaitingForReply(err: Error): void;
    flushAll(err: Error): void;
}
