/// <reference types="node" />
import EventEmitter from 'events';
import tls from 'tls';
interface RedisSocketCommonOptions {
    username?: string;
    password?: string;
    retryStrategy?(retries: number): number | Error;
}
interface RedisNetSocketOptions extends RedisSocketCommonOptions {
    port?: number;
    host?: string;
}
interface RedisUrlSocketOptions extends RedisSocketCommonOptions {
    url: string;
}
interface RedisUnixSocketOptions extends RedisSocketCommonOptions {
    path: string;
}
interface RedisTlsSocketOptions extends RedisNetSocketOptions {
    tls: tls.SecureContextOptions;
}
export declare type RedisSocketOptions = RedisNetSocketOptions | RedisUrlSocketOptions | RedisUnixSocketOptions | RedisTlsSocketOptions;
export declare type RedisSocketInitiator = () => Promise<void>;
export default class RedisSocket extends EventEmitter {
    #private;
    get isOpen(): boolean;
    get chunkRecommendedSize(): number;
    constructor(initiator?: RedisSocketInitiator, options?: RedisSocketOptions);
    connect(): Promise<void>;
    write(encodedCommands: string): boolean;
    disconnect(): Promise<void>;
}
export {};
