import { RedisClientType } from './client';
import { RedisModules } from './commands';
import { RedisLuaScripts } from './lua-script';
import { RedisSocketOptions } from './socket';
import { SinonSpy } from 'sinon';
import { RedisClusterType } from './cluster';
import { Context as MochaContext } from 'mocha';
declare type RedisVersion = [major: number, minor: number, patch: number];
declare type PartialRedisVersion = RedisVersion | [major: number, minor: number] | [major: number];
export declare function isRedisVersionGreaterThan(minimumVersion: PartialRedisVersion | undefined): boolean;
export declare enum TestRedisServers {
    OPEN = 0,
    PASSWORD = 1
}
export declare const TEST_REDIS_SERVERS: Record<TestRedisServers, RedisSocketOptions>;
export declare enum TestRedisClusters {
    OPEN = 0
}
export declare const TEST_REDIS_CLUSTERES: Record<TestRedisClusters, Array<RedisSocketOptions>>;
interface SpawnRedisServerResult {
    port: number;
    cleanup: () => Promise<void>;
}
export declare function spawnRedisCluster(type: TestRedisClusters | null, numberOfNodes: number, args?: Array<string>): Promise<Array<SpawnRedisServerResult>>;
export declare function spawnGlobalRedisCluster(type: TestRedisClusters | null, numberOfNodes: number, args?: Array<string>): Promise<Array<number>>;
interface RedisTestOptions {
    minimumRedisVersion?: PartialRedisVersion;
}
export declare function handleMinimumRedisVersion(mochaContext: MochaContext, minimumVersion: PartialRedisVersion | undefined): boolean;
export declare function describeHandleMinimumRedisVersion(minimumVersion: PartialRedisVersion): void;
export declare function itWithClient(type: TestRedisServers, title: string, fn: (client: RedisClientType<RedisModules, RedisLuaScripts>) => Promise<void>, options?: RedisTestOptions): void;
export declare function itWithCluster(type: TestRedisClusters, title: string, fn: (cluster: RedisClusterType<RedisModules, RedisLuaScripts>) => Promise<void>, options?: RedisTestOptions): void;
export declare function itWithDedicatedCluster(title: string, fn: (cluster: RedisClusterType<RedisModules, RedisLuaScripts>) => Promise<void>): void;
export declare function waitTillBeenCalled(spy: SinonSpy): Promise<void>;
export {};
